==> tinyca-0.4.3.tar.gz: TinyCA/lib/CA.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: CA.pm,v 1.3 2002/08/24 21:51:41 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CA;

sub new {
   my $that = shift;
   my $self = {};

   my $class = ref($that) || $that;

   $self->{'init'} = shift;

   if(not -d $self->{'init'}->{'basedir'}) {
      print "create basedir: $self->{'init'}->{'basedir'}\n";
      mkdir($self->{'init'}->{'basedir'}, 0700);
      mkdir($self->{'init'}->{'basedir'}."/tmp", 0700);
   }

   opendir(DIR, $self->{'init'}->{'basedir'}) || do {
      print "error: can't open basedir: $!";
      exit(1);
   };

   $self->{'calist'} = [];

      while(my $ca = readdir(DIR)) { 
         chomp($ca);
         next if $ca eq ".";
         next if $ca eq "..";
         next if $ca eq "tmp";

         my $dir = $self->{'init'}->{'basedir'}."/".$ca;
         next unless -d $dir;
         next unless -s $dir."/cacert.pem";
         next unless -s $dir."/cacert.key";
         push(@{$self->{'calist'}}, $ca);
         @{$self->{'calist'}} = sort(@{$self->{'calist'}});
         $self->{$ca}->{'dir'} = $dir;
         $self->{$ca}->{'cnf'} = $dir."/openssl.cnf";
      }
      closedir(DIR);

   bless($self, $class);
}

sub open_ca {
   my $self = shift;
   my $main  = shift;
   my $name = shift;

   my ($ind, $i, $cnf, @lines);

   if((not defined($name)) || ($name eq "")) {
      $ind = $main->show_open_dialog(\$name);
      return if not defined $ind;
      $name = $self->{'calist'}->[$ind];
   }

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   # selected CA is already open
   return if ((defined($self->{'actca'})) &&
              ($name eq $self->{'actca'}));

   $self->{'actca'} = $name;

   # update config 
   $cnf =  $self->{$name}->{'cnf'};
   open(IN, "<$cnf");
   @lines = <IN>;
   close(IN);
   for($i = 0; $lines[$i]; $i++) {
      $lines[$i] =~ s/private\/cakey.pem/cacert.key/;
   }
   open(OUT, ">$cnf");
   print OUT @lines;
   close(OUT);

   $main->{'mw'}->configure(
         -title => "Tiny CA Management $main->{'version'}".
                   " - $self->{'actca'}"
                   );

   if(defined($main->{'catab'})) {
      $main->{'catab'}->destroy();
      delete($main->{'catab'});
   }
   if(defined($main->{'certtab'})) {
      $main->{'certtab'}->destroy();
      delete($main->{'certtab'});
   }
   if(defined($main->{'reqtab'})) {
      $main->{'reqtab'}->destroy();
      delete($main->{'reqtab'});
   }
   if(defined($main->{'keytab'})) {
      $main->{'keytab'}->destroy();
      delete($main->{'keytab'});
   }

   $main->{'CERT'}->{'lastread'} = 0;
   $main->{'REQ'}->{'lastread'}  = 0;
   $main->{'KEY'}->{'lastread'}  = 0;

   $main->{'Openssl'} = OpenSSL->new($main);

   $main->act_menu();
   $main->create_mframe();

   return;
}

sub delete_ca {
   my $self = shift;
   my $main = shift;

   my ($ind, @tmp, $name);

   $ind = $main->show_del_dialog(\$name);
   return if not defined $ind;
   $name = $self->{'calist'}->[$ind];

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   _rm_dir($self->{$name}->{'dir'});

   if((defined($self->{'actca'})) && 
      ($name eq $self->{'actca'})) { 
      $self->{'actca'}   = undef;
      $main->{'OpenSSL'} = undef;
   }
   
   $main->{'catab'}->destroy() if(defined($main->{'catab'}));
   delete($main->{'catab'});

   $main->{'reqtab'}->destroy() if(defined($main->{'reqtab'}));
   delete($main->{'reqtab'});

   $main->{'keytab'}->destroy() if(defined($main->{'keytab'}));
   delete($main->{'keytab'});

   $main->{'certtab'}->destroy() if(defined($main->{'certtab'}));
   delete($main->{'certtab'});

   $main->{'nb'}->destroy() if(defined($main->{'nb'}));
   delete($main->{'nb'});

   $main->{'mframe'}->destroy() if(defined($main->{'mframe'}));
   delete($main->{'mframe'});

   foreach(@{$self->{'calist'}}) {
      next if $_ eq $name;
      push(@tmp, $_);
   }
   $self->{'calist'} = \@tmp;


   delete($self->{$name});

   $main->create_mframe();

   return;
}

sub create_ca {
   my $self      = shift;
   my $main      = shift;

   my ($opts, $name, $action, $index, $serial);
   $opts = {};
   $opts->{'days'} = 3650; # set default to 10 years
   $opts->{'bits'} = 2048;

   $action = $main->show_ca_dialog($opts);

   return if $action eq "Cancel";

   while((not defined($opts->{'name'})) || 
         ($opts->{'name'} eq "") ||
         ($opts->{'name'} =~ /\s/)) {
      $main->print_warning("Name must be filled in and must"
                           ." not contain Spaces");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   };

   while((not defined($opts->{'passwd'}) || 
          not defined($opts->{'passwd2'})) ||
         $opts->{'passwd'} ne $opts->{'passwd2'}) {
      $main->print_warning("Passwords don't match");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   while(not defined($opts->{'C'}) ||
         length($opts->{'C'}) != 2) {
      $main->print_warning("Country must be exact 2 letter code");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   while((not defined($opts->{'C'})) ||
         ($opts->{'C'} eq "") ||
         (not defined($opts->{'CN'})) ||
         ($opts->{'CN'} eq "")) {
      $main->print_warning("Please fill in at least CN, C and Passwd field");
      $action = $main->show_ca_dialog($opts);
      return if $action eq "Cancel";
   }

   $name = $opts->{'name'};

   if(defined($self->{$name})) {
      $main->print_warning("CA: $name already exists");
      return;
   }

   # create directories
   $self->{$name}->{'dir'} = $self->{'init'}->{'basedir'}."/".$name;

   mkdir($self->{$name}->{'dir'}, 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/req", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/keys", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/certs", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/crl", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/newcerts", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   # create configuration file
   my $in  = $self->{'init'}->{'templatedir'}."/openssl.cnf";
   my $out = $self->{$name}->{'dir'}."/openssl.cnf";

   open(IN, "<$in") || do {
      $main->print_error("can't open template: $in $!");
      return;
   };
   open(OUT, ">$out") || do {
      $main->print_error("can't open outfile: $!");
      return;
   };
   while(<IN>) {
      s/\%dir\%/$self->{$name}->{'dir'}/;
      print OUT;
   }
   close IN;
   close OUT;
   $self->{$name}->{'cnf'} = $out;

   # create some more files
   $index = $self->{$name}->{'dir'}."/index.txt";
   open(OUT, ">$index") || do {
      $main->print_error("can't open indexfile: $!");
      return;
   };
   close OUT;

   $serial = $self->{$name}->{'dir'}."/serial";
   open(OUT, ">$serial") || do {
      $main->print_error("can't open serialfile: $!");
      return;
   };
   print OUT "01";
   close OUT;

   # initial configuration
   $action = $main->show_config_dialog($name);

   if($action eq 'Cancel') {
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }
   
   # create CA certifikate
   $main->{'OpenSSL'}->newkey( 
         'bits'    => $opts->{'bits'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.key",
         'pass'    => $opts->{'passwd'}
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.key") {
      $main->print_warning("newkey() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   $main->{'OpenSSL'}->newreq( 
         'config'  => $self->{$name}->{'cnf'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.req",
         'pass'    => $opts->{'passwd'},
         'dn'      => [ $opts->{'C'}, 
                        $opts->{'ST'},
                        $opts->{'L'},
                        $opts->{'O'},
                        $opts->{'OU'},
                        $opts->{'CN'},
                        $opts->{'EMAIL'},
                        '',
                        ''
                        ],
         'keyfile' => $self->{$name}->{'dir'}."/cacert.key"
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.req") {
      unlink($self->{$name}->{'dir'}."/cacert.key");
      unlink($self->{$name}->{'dir'}."/cacert.req");
      $main->print_warning("newreq() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   $main->{'OpenSSL'}->newcert( 
         'config'  => $self->{$name}->{'cnf'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.pem",
         'keyfile' => $self->{$name}->{'dir'}."/cacert.key",
         'reqfile' => $self->{$name}->{'dir'}."/cacert.req",
         'pass'    => $opts->{'passwd'},
         'days'    => $opts->{'days'}
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.pem") {
      $main->print_warning("newcert() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   unlink($self->{$name}->{'dir'}."/cacert.req");

   $main->{'OpenSSL'}->newcrl(
         config  => $self->{$name}->{'cnf'},
         pass    => $opts->{'passwd'},
         crldays => 365,
         outfile => $self->{$name}->{'dir'}."/crl/crl.pem",
         format  => 'PEM'
         );

   if (not -s $self->{$name}->{'dir'}."/crl/crl.pem") {
      $main->print_warning("newcrl() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   # seems to be done
   push(@{$self->{'calist'}}, $name);
   @{$self->{'calist'}} = sort(@{$self->{'calist'}});
   $main->print_info("CA: $name created");

   $self->open_ca($main, $name);
   return;
}

sub export_ca_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $format, $out, $keyfile, $parsed);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";

   $action = $main->show_ca_export_dialog(
         \$outfile,
         \$format
         );

   return if $action eq "Cancel";
   return if(not defined($outfile));

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');

   if(not defined $parsed) {
      $main->print_error("can't read CA Certificate");
   }

   if($format eq "PEM") {
      $out = $parsed->{'PEM'};
   } elsif ($format eq "DER") {
      $out = $parsed->{'DER'};
   } elsif ($format eq "TXT") {
      $out = $parsed->{'TEXT'};
   } else {
      $main->print_warning("Invalid Format");
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning("Can't open Outfile!");
      return;
   };

   print OUT $out;
   close OUT;
   
   $main->print_info("Certificate exported to: $outfile");

   return;
}

sub export_crl {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $passwd,
      $format, $x509, $out, $keyfile);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";
   $outfile = "/tmp/crl.pem";

   $action = $main->show_crl_export_dialog(
         \$outfile,
         \$format,
         \$passwd
         );

   return if $action eq "Cancel";

   while((not defined($outfile)) || 
         ($outfile eq "")) {
      $main->print_warning("No outfile selected");
      $action = $main->show_crl_export_dialog(
            \$outfile,
            \$format,
            \$passwd
            );
      return if $action eq "Cancel";
   };

   while((not defined($passwd)) || 
         ($passwd eq "")) {
      $main->print_warning("No Password given");
      $action = $main->show_crl_export_dialog(
            \$outfile,
            \$format,
            \$passwd
            );
      return if $action eq "Cancel";
   };

   if(not defined($main->{'OpenSSL'})) {
      $main->init_openssl($ca);
   }

   $main->{'OpenSSL'}->newcrl(
         config  => $self->{$ca}->{'cnf'},
         pass    => $passwd,
         crldays => 365,
         outfile => $outfile,
         format  => $format
         );

   if (not -s $outfile) {
      $main->print_error("newcrl() failed");
   }

   $main->print_info("Crl exported to: $outfile");

   return;
}

sub _get_date {
   my $string = shift;

   $string =~ s/  / /g;

   my @t1 = split(/ /, $string);
   my @t2 = split(/:/, $t1[2]);

      $t1[0] = _get_index($t1[0]);

   my $ret = Time::Local::timelocal($t2[2],$t2[1],$t2[0],$t1[1],$t1[0],$t1[3]);

   return($ret);
}

sub _get_index {
   my $m = shift;
   my @a = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

   for(my $i = 0; $a[$i]; $i++) {
      return $i if($a[$i] eq $m);
   }
}

sub _rm_dir {
   my $dir = shift;

   my $dirh;

   opendir($dirh, $dir);

   while(my $f = readdir($dirh)) {
      next if $f eq '.';
      next if $f eq '..';

      if(-d $dir."/".$f) {
         _rm_dir($dir."/".$f);
      } else {
         unlink($dir."/".$f);
      }
   }
   closedir(DIR);

   rmdir($dir);
   
   return(0);
}

1

==> tinyca-0.4.6.tar.gz: TinyCA/lib/CA.pm <==
# Copyright (c) Stephan Martin <sm@sm-zone.net>
#
# $Id: CA.pm,v 1.9 2003/06/19 21:46:43 sm Exp $
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.

use strict;

package CA;

sub new {
   my $that = shift;
   my $self = {};

   my $class = ref($that) || $that;

   $self->{'init'} = shift;

   if(not -d $self->{'init'}->{'basedir'}) {
      print "create basedir: $self->{'init'}->{'basedir'}\n";
      mkdir($self->{'init'}->{'basedir'}, 0700);
   }

   if(not -d $self->{'init'}->{'basedir'}."/tmp") {
      print "create temp dir: $self->{'init'}->{'basedir'}/tmp\n";
      mkdir($self->{'init'}->{'basedir'}."/tmp", 0700);
   }

   opendir(DIR, $self->{'init'}->{'basedir'}) || do {
      print "error: can't open basedir: $!";
      exit(1);
   };

   $self->{'calist'} = [];

      while(my $ca = readdir(DIR)) { 
         chomp($ca);
         next if $ca eq ".";
         next if $ca eq "..";
         next if $ca eq "tmp";

         my $dir = $self->{'init'}->{'basedir'}."/".$ca;
         next unless -d $dir;
         next unless -s $dir."/cacert.pem";
         next unless -s $dir."/cacert.key";
         push(@{$self->{'calist'}}, $ca);
         @{$self->{'calist'}} = sort(@{$self->{'calist'}});
         $self->{$ca}->{'dir'} = $dir;
         $self->{$ca}->{'cnf'} = $dir."/openssl.cnf";
      }
      closedir(DIR);

   bless($self, $class);
}

sub open_ca {
   my $self = shift;
   my $main  = shift;
   my $name = shift;

   my ($ind, $i, $cnf, @lines);

   if((not defined($name)) || ($name eq "")) {
      # if only one CA is defined open it without prompting
      if($#{$self->{'calist'}} == 0) {
	 $name = $self->{'calist'}->[0];
      }
      else {
	 $ind = $main->show_open_dialog(\$name);
	 return if not defined $ind;
	 $name = $self->{'calist'}->[$ind];
      }
   }

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   # selected CA is already open
   return if ((defined($self->{'actca'})) &&
              ($name eq $self->{'actca'}));

   $self->{'actca'} = $name;

   # update config (necessary for update from old tinyca)
   $cnf =  $self->{$name}->{'cnf'};
   open(IN, "<$cnf");
   @lines = <IN>;
   close(IN);
   for($i = 0; $lines[$i]; $i++) {
      $lines[$i] =~ s/private\/cakey.pem/cacert.key/;
   }
   open(OUT, ">$cnf");
   print OUT @lines;
   close(OUT);

   $main->{'mw'}->configure(
         -title => "Tiny CA Management $main->{'version'}".
                   " - $self->{'actca'}"
                   );

   if(defined($main->{'catab'})) {
      $main->{'catab'}->destroy();
      delete($main->{'catab'});
   }
   if(defined($main->{'certtab'})) {
      $main->{'certtab'}->destroy();
      delete($main->{'certtab'});
   }
   if(defined($main->{'reqtab'})) {
      $main->{'reqtab'}->destroy();
      delete($main->{'reqtab'});
   }
   if(defined($main->{'keytab'})) {
      $main->{'keytab'}->destroy();
      delete($main->{'keytab'});
   }

   $main->{'CERT'}->{'lastread'} = 0;
   $main->{'REQ'}->{'lastread'}  = 0;
   $main->{'KEY'}->{'lastread'}  = 0;

   $main->{'Openssl'} = OpenSSL->new($main);

   # could be necessary during update from openssl 0.9.6 to 0.9.7
#$index = $main->{'CA'}->{$ca}->{'dir'}."/index.txt";

   $main->{'CONFIG'}->init_config($main, $name);

   $main->act_menu();
   $main->create_mframe();

   return;
}

sub delete_ca {
   my $self = shift;
   my $main = shift;

   my ($ind, @tmp, $name);

   $ind = $main->show_del_dialog(\$name);
   return if not defined $ind;
   $name = $self->{'calist'}->[$ind];

   if(!exists($self->{$name})) {
      $main->print_warning("Invalid CA selected");
      return;
   }

   _rm_dir($self->{$name}->{'dir'});

   if((defined($self->{'actca'})) && 
      ($name eq $self->{'actca'})) { 
      $self->{'actca'}   = undef;
#      $main->{'OpenSSL'} = undef;
   }
   
   $main->{'catab'}->destroy() if(defined($main->{'catab'}));
   delete($main->{'catab'});

   $main->{'reqtab'}->destroy() if(defined($main->{'reqtab'}));
   delete($main->{'reqtab'});

   $main->{'keytab'}->destroy() if(defined($main->{'keytab'}));
   delete($main->{'keytab'});

   $main->{'certtab'}->destroy() if(defined($main->{'certtab'}));
   delete($main->{'certtab'});

   $main->{'nb'}->destroy() if(defined($main->{'nb'}));
   delete($main->{'nb'});

   $main->{'mframe'}->destroy() if(defined($main->{'mframe'}));
   delete($main->{'mframe'});

   foreach(@{$self->{'calist'}}) {
      next if $_ eq $name;
      push(@tmp, $_);
   }
   $self->{'calist'} = \@tmp;


   delete($self->{$name});

   $main->act_menu();
   $main->create_mframe();

   return;
}

sub create_ca {
   my $self      = shift;
   my $main      = shift;

   my ($opts, $name, $action, $index, $serial);
   $opts = {};
   $opts->{'days'} = 3650; # set default to 10 years
   $opts->{'bits'} = 2048;

   {
      $action = $main->show_ca_dialog($opts);

      return if $action eq "Cancel";

      if((not defined($opts->{'name'})) || 
	    ($opts->{'name'} eq "") ||
	    ($opts->{'name'} =~ /\s/)) {
	 $main->print_warning("Name must be filled in and must"
                        ." not contain Spaces");
	 redo;
      };

      if((not defined($opts->{'C'})) ||
         ($opts->{'C'} eq "") ||
         (not defined($opts->{'CN'})) ||
         ($opts->{'CN'} eq "") ||
         (not defined($opts->{'passwd'})) ||
         ($opts->{'passwd'} eq "")) {
        $main->print_warning("Please specify at least Common Name, "
                            ."Country and Password");
        redo;
      }

      if((not defined($opts->{'passwd2'})) ||
          $opts->{'passwd'} ne $opts->{'passwd2'}) {
         $main->print_warning("Passwords don't match");
         redo;
      }

      if(length($opts->{'C'}) != 2) {
         $main->print_warning("Country must be exact 2 letter code");
         redo;
      }

      $name = $opts->{'name'};

      if(defined($self->{$name})) {
         $main->print_warning("CA: $name already exists");
         redo;
      }
   }

   # create directories
   $self->{$name}->{'dir'} = $self->{'init'}->{'basedir'}."/".$name;

   mkdir($self->{$name}->{'dir'}, 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/req", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/keys", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/certs", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/crl", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   mkdir($self->{$name}->{'dir'}."/newcerts", 0700) || do { 
      $main->print_error("can't create directory: $!");
      return;
   };

   # create configuration file
   my $in  = $self->{'init'}->{'templatedir'}."/openssl.cnf";
   my $out = $self->{$name}->{'dir'}."/openssl.cnf";

   open(IN, "<$in") || do {
      $main->print_error("can't open template: $in $!");
      return;
   };
   open(OUT, ">$out") || do {
      $main->print_error("can't open outfile: $!");
      return;
   };
   while(<IN>) {
      s/\%dir\%/$self->{$name}->{'dir'}/;
      print OUT;
   }
   close IN;
   close OUT;
   $self->{$name}->{'cnf'} = $out;

   $main->{'CONFIG'}->init_config($main, $name);

   # create some more files
   $index = $self->{$name}->{'dir'}."/index.txt";
   open(OUT, ">$index") || do {
      $main->print_error("can't open indexfile: $!");
      return;
   };
   close OUT;

   $serial = $self->{$name}->{'dir'}."/serial";
   open(OUT, ">$serial") || do {
      $main->print_error("can't open serialfile: $!");
      return;
   };
   print OUT "01";
   close OUT;

   # initial configuration
   $action = $main->show_config_dialog($name);

   if($action eq 'Cancel') {
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }
   
   # create CA certifikate
   $main->{'OpenSSL'}->newkey( 
         'bits'    => $opts->{'bits'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.key",
         'pass'    => $opts->{'passwd'}
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.key") {
      $main->print_warning("newkey() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   $main->{'OpenSSL'}->newreq( 
         'config'  => $self->{$name}->{'cnf'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.req",
         'pass'    => $opts->{'passwd'},
         'dn'      => [ $opts->{'C'}, 
                        $opts->{'ST'},
                        $opts->{'L'},
                        $opts->{'O'},
                        $opts->{'OU'},
                        $opts->{'CN'},
                        $opts->{'EMAIL'},
                        '',
                        ''
                        ],
         'keyfile' => $self->{$name}->{'dir'}."/cacert.key"
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.req") {
      unlink($self->{$name}->{'dir'}."/cacert.key");
      unlink($self->{$name}->{'dir'}."/cacert.req");
      $main->print_warning("newreq() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   $main->{'OpenSSL'}->newcert( 
         'config'  => $self->{$name}->{'cnf'},
         'outfile' => $self->{$name}->{'dir'}."/cacert.pem",
         'keyfile' => $self->{$name}->{'dir'}."/cacert.key",
         'reqfile' => $self->{$name}->{'dir'}."/cacert.req",
         'pass'    => $opts->{'passwd'},
         'days'    => $opts->{'days'}
         );
   
   if (not -s $self->{$name}->{'dir'}."/cacert.pem") {
      $main->print_warning("newcert() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   unlink($self->{$name}->{'dir'}."/cacert.req");

   $main->{'OpenSSL'}->newcrl(
         $main,
         config  => $self->{$name}->{'cnf'},
         pass    => $opts->{'passwd'},
         crldays => $main->{'CONFIG'}->{'server_ca'}->{'default_crl_days'},
         outfile => $self->{$name}->{'dir'}."/crl/crl.pem",
         format  => 'PEM'
         );

   if (not -s $self->{$name}->{'dir'}."/crl/crl.pem") {
      $main->print_warning("newcrl() failed");
      _rm_dir($self->{$name}->{'dir'});
      delete($self->{$name});
      return;
   }

   # seems to be done
   push(@{$self->{'calist'}}, $name);
   @{$self->{'calist'}} = sort(@{$self->{'calist'}});
   $main->print_info("CA: $name created");

   $self->open_ca($main, $name);
   return;
}

sub export_ca_cert {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $format, $out, $keyfile, $parsed);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";

   $action = $main->show_ca_export_dialog(
         \$outfile,
         \$format
         );

   return if $action eq "Cancel";
   return if(not defined($outfile));

   $parsed = $main->{'CERT'}->parse_cert($main, 'CA');

   if(not defined $parsed) {
      $main->print_error("can't read CA Certificate");
   }

   if($format eq "PEM") {
      $out = $parsed->{'PEM'};
   } elsif ($format eq "DER") {
      $out = $parsed->{'DER'};
   } elsif ($format eq "TXT") {
      $out = $parsed->{'TEXT'};
   } else {
      $main->print_warning("Invalid Format");
      return;
   }

   open(OUT, ">$outfile") || do {
      $main->print_warning("Can't open Outfile!");
      return;
   };

   print OUT $out;
   close OUT;
   
   $main->print_info("Certificate exported to: $outfile");

   return;
}

sub export_crl {
   my $self = shift;
   my $main = shift;
   my $ca   = $self->{'actca'};
    
   my($certfile, $outfile, $action, $passwd,
      $format, $x509, $out, $keyfile, $days);

   $certfile = $self->{$ca}->{'dir'}."/cacert.pem";
   $outfile = "/tmp/crl.pem";

   $days = $main->{'CONFIG'}->{'server_ca'}->{'default_crl_days'};

   {
      $action = $main->show_crl_export_dialog(
	    \$outfile,
	    \$format,
	    \$passwd,
	    \$days
	    );

      return if $action eq "Cancel";

      if((not defined($outfile)) || 
	    ($outfile eq "")) {
	 $main->print_warning("No output file selected");
	 redo;
      };

      if((not defined($passwd)) || 
	    ($passwd eq "")) {
	 $main->print_warning("No Password given");
	 redo;
      };
   }

   if(not defined($main->{'OpenSSL'})) {
      # XXX doesn't exist!?
      $main->init_openssl($ca);
   }

   $main->{'OpenSSL'}->newcrl(
         $main,
         config  => $self->{$ca}->{'cnf'},
         pass    => $passwd,
         crldays => $days,
         outfile => $outfile,
         format  => $format
         );

   if (not -s $outfile) {
      $main->print_warning("newcrl() failed");
      return;
   }

   $main->print_info("Crl exported to: $outfile");

   return;
}

sub _get_date {
   my $string = shift;

   $string =~ s/  / /g;

   my @t1 = split(/ /, $string);
   my @t2 = split(/:/, $t1[2]);

      $t1[0] = _get_index($t1[0]);

   my $ret = Time::Local::timelocal($t2[2],$t2[1],$t2[0],$t1[1],$t1[0],$t1[3]);

   return($ret);
}

sub _get_index {
   my $m = shift;
   my @a = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

   for(my $i = 0; $a[$i]; $i++) {
      return $i if($a[$i] eq $m);
   }
}

sub _rm_dir {
   my $dir = shift;

   my $dirh;

   opendir($dirh, $dir);

   while(my $f = readdir($dirh)) {
      next if $f eq '.';
      next if $f eq '..';

      if(-d $dir."/".$f) {
         _rm_dir($dir."/".$f);
      } else {
         unlink($dir."/".$f);
      }
   }
   closedir(DIR);

   rmdir($dir);
   
   return(0);
}

1
